package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.command.handling.*;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.JoinConfiguration;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.List;

@Intercept("list")
@CommandPermissions(permission = "list", source = SourceType.BOTH)
@CommandParameters(description = "Lists the real names of all online players.", usage = "/<command> [-a | -v]", aliases = "who,lsit")
public class Command_list extends FreedomCommand
{
    @Override
    public boolean run(final CommandSender sender, final Player playerSender, final Command cmd, final String commandLabel, final String[] args, final boolean senderIsConsole)
    {
        if (args.length > 1)
        {
            return false;
        }

        ListFilter listFilter;
        if (args.length == 1)
        {
            switch (args[0].toLowerCase())
            {
                case "-s", "-a" -> listFilter = ListFilter.ADMINS;
                case "-v" ->
                {
                    checkPermission("tfm.list.vanished");
                    listFilter = ListFilter.VANISHED_ADMINS;
                }
                case "-t" ->
                {
                    checkPermission("tfm.list.telnet");
                    listFilter = ListFilter.TELNET_SESSIONS;
                }
                default ->
                {
                    return false;
                }
            }
        } else
        {
            listFilter = ListFilter.PLAYERS;
        }

        List<TextComponent> players;

        if (listFilter == ListFilter.TELNET_SESSIONS && plugin.al.isAdmin(sender))
        {
            players = plugin.btb.getConnectedAdmins().stream().map(admin -> Component.text(admin.getName())).toList();
            msgNew("<blue>There are <red><count></red> admins connected to telnet.",
                    Placeholder.unparsed("count", String.valueOf(players.size())));
        } else
        {
            players = server.getOnlinePlayers().stream().filter(pl ->
                    (listFilter == ListFilter.ADMINS && plugin.al.isAdmin(pl) && !plugin.al.isVanished(pl.getUniqueId()))
                            || (listFilter == ListFilter.VANISHED_ADMINS && plugin.al.isVanished(pl.getUniqueId()))
                            || (listFilter == ListFilter.PLAYERS && !plugin.al.isVanished(pl.getUniqueId()))).map(player ->
                    Component.textOfChildren(plugin.rm.getDisplay(player).getColoredTag().append(Component.text(player.getName())))).toList();

            msgNew("<blue>There are <red><count></red> out of a maximum of <red><max></red> players online.",
                    Placeholder.unparsed("count", String.valueOf(FUtil.getFakePlayerCount())),
                    Placeholder.unparsed("max", String.valueOf(server.getMaxPlayers())));
        }

        msgNew("Connected <type>: <players>",
                Placeholder.unparsed("type", listFilter.name().toLowerCase().replace('_', ' ')),
                Placeholder.component("players", Component.join(JoinConfiguration.commas(true), players)));
        return true;
    }

    private enum ListFilter
    {
        PLAYERS,
        ADMINS,
        VANISHED_ADMINS,
        TELNET_SESSIONS
    }
}