package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.admin.Admin;
import me.totalfreedom.totalfreedommod.command.handling.CommandParameters;
import me.totalfreedom.totalfreedommod.command.handling.CommandPermissions;
import me.totalfreedom.totalfreedommod.command.handling.FreedomCommand;
import me.totalfreedom.totalfreedommod.command.handling.SourceType;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.FPlayer;
import me.totalfreedom.totalfreedommod.rank.DisplayableGroup;
import me.totalfreedom.totalfreedommod.rank.GroupProvider;
import me.totalfreedom.totalfreedommod.rank.Hierarchy;
import me.totalfreedom.totalfreedommod.util.FLog;
import me.totalfreedom.totalfreedommod.util.FUtil;
import me.totalfreedom.totalfreedommod.util.PermissibleCompletion;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.luckperms.api.model.group.Group;
import org.apache.commons.lang.StringUtils;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@CommandPermissions(permission = "saconfig", source = SourceType.BOTH)
@CommandParameters(description = "List, add, remove, or set the rank of admins, clean or reload the admin list, or view admin information.", usage = "/<command> <list | clean | reload | | setrank <username> <rank> | <add | remove | info> <username>>", aliases = "slconfig")
public class Command_saconfig extends FreedomCommand
{

    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (args.length < 1)
        {
            return false;
        }

        switch (args[0])
        {
            case "list" ->
            {
                msgNew("<gold>Admins: <admins>", Placeholder.unparsed("admins", StringUtils.join(plugin.al.getAdminNames(), ", ")));
                return true;
            }
            case "clean" ->
            {
                checkConsole();
                checkPermission("tfm.saconfig.clean");

                FUtil.adminAction(sender.getName(), "Cleaning the admin list", true);
                plugin.al.deactivateOldEntries(true);
                msgNew("<gold>Admins: <admins>", Placeholder.unparsed("admins", StringUtils.join(plugin.al.getAdminNames(), ", ")));
                return true;
            }
            case "reload" ->
            {
                checkPermission("tfm.saconfig.reload");

                FUtil.adminAction(sender.getName(), "Reloading the admin list", true);
                plugin.al.load();
                msgNew("Admin list reloaded!");
                return true;
            }
            case "promote" ->
            {
                checkConsole();
                checkPermission("tfm.saconfig.promote");

                if (args.length < 2)
                {
                    return false;
                }

                Player player = getPlayer(args[1]);

                if (player == null)
                {
                    msgNew("<red>Player not found.");
                    return true;
                }

                Hierarchy.getHierarchy()
                        .promoteUser(Hierarchy.getHierarchy().admin(), player)
                        .whenComplete((result, ex) ->
                        {
                            if (ex != null)
                            {
                                FLog.severe("Failed to promote "
                                        + player.getName()
                                        + " to the next rank");
                                return;
                            }

                            if (result == null) {
                                msgNew("<green>Added <player> to the ADMIN group.", player(player));
                            }
                            else if (!result.wasSuccessful() || result.getGroupTo().isEmpty())
                            {
                                msgNew("<red>Failed to promote <player> to the next rank.", player(player));
                                return;
                            } else {
                                result.getGroupTo().ifPresentOrElse(group ->
                                {
                                    Admin admin = plugin.al.getAdmin(player);

                                    if (admin == null)
                                    {
                                        admin = new Admin(player);
                                        plugin.al.addAdmin(admin);
                                    }

                                    Group actual = Hierarchy.getHierarchy().getGroup(group);
                                    DisplayableGroup rank = GroupProvider.fromLuckPermsGroup(actual).getGroup();

                                    if (rank == null)
                                    {
                                        msgNew("<red>Unknown rank: <rank>", Placeholder.unparsed("rank", args[2]));
                                        return;
                                    }

                                    admin.setRank(rank);
                                    admin.addIp(FUtil.getIp(player));
                                    admin.setActive(true);
                                    admin.setLastLogin(new Date());

                                    plugin.al.save(admin);
                                    plugin.al.updateTables();

                                    if (plugin.dc != null && plugin.dc.isEnabled() && ConfigEntry.DISCORD_ROLE_SYNC.getBoolean())
                                    {
                                        plugin.dc.syncRoles(admin, plugin.pl.getData(player).getDiscordID());
                                    }

                                    msgNew("Set <admin>'s rank to <rank>.", Placeholder.unparsed("admin", admin.getName()), Placeholder.component("rank", rank.getName()));

                                    final FPlayer fPlayer = plugin.pl.getPlayer(player);
                                    if (fPlayer.getFreezeData().isFrozen())
                                    {
                                        fPlayer.getFreezeData().setFrozen(false);
                                        msgNew(player, "You have been unfrozen.");
                                    }

                                    msgNew("<gold>Admin <player> has been promoted to <rank>",
                                            player(player),
                                            Placeholder.unparsed("rank", group));
                                }, () -> msgNew("<red>Failed to promote <player> to the next rank.", player(player)));
                            }
                        });

                return true;
            }
            case "info" ->
            {
                if (args.length < 2)
                {
                    return false;
                }

                checkPermission("tfm.saconfig.info");

                Admin admin = plugin.al.getEntryByName(args[1]);

                if (admin == null)
                {
                    final Player player = getPlayer(args[1]);
                    if (player != null)
                    {
                        admin = plugin.al.getAdmin(player);
                    }
                }

                if (admin == null)
                {
                    msgNew("Unknown admin: <arg>", arg(args[1]));
                    return true;
                } else
                {
                    msgNew(admin.toString());
                }

                return true;
            }
            case "demote" ->
            {
                checkConsole();
                checkPermission("tfm.saconfig.demote");

                if (args.length < 2)
                {
                    return false;
                }

                Player player = getPlayer(args[1]);

                if (player == null)
                {
                    msgNew("<red>Player not found.");
                    return true;
                }

                Hierarchy.getHierarchy()
                        .demoteUser(Hierarchy.getHierarchy().admin(), player)
                        .whenComplete((result, ex) ->
                        {
                            if (ex != null)
                            {
                                FLog.severe("Failed to demote "
                                        + player.getName()
                                        + " to the next rank");
                                return;
                            }

                            if (result == null || !result.wasSuccessful() || result.getGroupTo().isEmpty())
                            {
                                msgNew("<red>Failed to demote <player> to the next rank.", player(player));
                                return;
                            }

                            result.getGroupTo().ifPresentOrElse(group ->
                            {
                                Admin admin = plugin.al.getAdmin(player);

                                if (admin == null)
                                {
                                    admin = new Admin(player);
                                    plugin.al.addAdmin(admin);
                                }

                                Group actual = Hierarchy.getHierarchy().getGroup(group);
                                DisplayableGroup rank = GroupProvider.fromLuckPermsGroup(actual).getGroup();

                                if (rank == null)
                                {
                                    msgNew("<red>Unknown rank: <rank>", Placeholder.unparsed("rank", args[2]));
                                    return;
                                }

                                admin.setRank(rank);
                                admin.setLastLogin(new Date());

                                plugin.al.save(admin);
                                plugin.al.updateTables();

                                if (plugin.dc != null && plugin.dc.isEnabled() && ConfigEntry.DISCORD_ROLE_SYNC.getBoolean())
                                {
                                    plugin.dc.syncRoles(admin, plugin.pl.getData(player).getDiscordID());
                                }

                                msgNew("Set <admin>'s rank to <rank>.", Placeholder.unparsed("admin", admin.getName()), Placeholder.component("rank", rank.getName()));

                                final FPlayer fPlayer = plugin.pl.getPlayer(player);
                                if (fPlayer.getFreezeData().isFrozen())
                                {
                                    fPlayer.getFreezeData().setFrozen(false);
                                    msgNew(player, "You have been unfrozen.");
                                }

                                msgNew("<gold>Admin <player> has been demoted to <rank>",
                                        Placeholder.unparsed("player", player.getName()),
                                        Placeholder.unparsed("rank", group));
                            }, () -> msgNew("<red>Failed to demote <player> to the next rank.", Placeholder.unparsed("player", player.getName())));
                        });
            }
            case "remove" ->
            {
                if (args.length < 2)
                {
                    return false;
                }

                checkConsole();
                checkPermission("tfm.saconfig.remove");

                Player player = getPlayer(args[1]);
                Admin admin = plugin.al.getAdmin(player);
                plugin.al.removeAdmin(admin);

                Hierarchy.getHierarchy().dropUserFromAll(Hierarchy.getHierarchy().admin(), player);

                FUtil.adminAction(sender.getName(), "Removing " + admin.getName() + " from the admin list", true);
                admin.setActive(false);

                plugin.al.save(admin);
                plugin.al.updateTables();

                if (plugin.dc != null && plugin.dc.isEnabled() && ConfigEntry.DISCORD_ROLE_SYNC.getBoolean())
                {
                    plugin.dc.syncRoles(admin, plugin.pl.getData(admin.getName())
                            .getDiscordID());
                }

                return true;
            }
            default ->
            {
                return false;
            }
        }
        return false;
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (args.length == 1)
        {
            return argumentCompletions(args[0],
                    PermissibleCompletion.of("tfm.saconfig.info", "info"),
                    PermissibleCompletion.of("tfm.saconfig.demote", "demote"),
                    PermissibleCompletion.of("tfm.saconfig.remove", "remove"),
                    PermissibleCompletion.of("tfm.saconfig.reload", "reload"),
                    PermissibleCompletion.of("tfm.saconfig.clean", "clean"),
                    PermissibleCompletion.of("tfm.saconfig.promote", "promote"));
        }
        if (args.length == 2 && (args[0].equals("add") || args[0].equals("remove") || args[0].equals("setrank") || args[0].equals("info")))
        {
            return playerCompletions(args[1]);
        }
        if (args.length == 3 && args[0].equals("setrank") && sender.hasPermission("tfm.saconfig.setrank"))
        {
            return Arrays.asList("ADMIN", "SENIOR_ADMIN");
        }

        return Collections.emptyList();
    }
}
