package me.totalfreedom.totalfreedommod.command.handling;

import me.totalfreedom.totalfreedommod.FreedomService;
import me.totalfreedom.totalfreedommod.util.FLog;
import org.bukkit.Bukkit;
import org.reflections.Reflections;

import java.util.*;
import java.util.concurrent.CompletableFuture;

public class CommandLoader extends FreedomService
{
    private final List<FreedomCommand> commands;

    public CommandLoader()
    {
        commands = new ArrayList<>();
    }

    @Override
    public void onStart()
    {
    }

    @Override
    public void onStop()
    {
    }

    public void add(FreedomCommand command)
    {
        commands.add(command);
        command.register();
    }

    public FreedomCommand getByName(String name)
    {
        for (FreedomCommand command : commands)
        {
            if (name.equals(command.getName()))
            {
                return command;
            }
        }
        return null;
    }

    public boolean isAlias(String alias)
    {
        for (FreedomCommand command : commands)
        {
            if (Arrays.asList(command.getAliases().split(",")).contains(alias))
            {
                return true;
            }
        }
        return false;
    }

    public void loadCommands()
    {
        Reflections commandDir = new Reflections("me.totalfreedom.totalfreedommod.command");

        Interceptor interceptor = new Interceptor(plugin);

        Set<Class<? extends FreedomCommand>> commandClasses = commandDir.getSubTypesOf(FreedomCommand.class);

        Set<Class<? extends FreedomCommand>> postLoadCommands = new HashSet<>();

        for (Class<? extends FreedomCommand> commandClass : commandClasses)
        {
            if (commandClass.isAnnotationPresent(CommandDependencies.class))
            {
                postLoadCommands.add(commandClass);
            }

            if (commandClass.isAnnotationPresent(Intercept.class))
            {
                Intercept intercept = commandClass.getAnnotation(Intercept.class);
                interceptor.intercept(intercept.value());
                postLoadCommands.add(commandClass);
            }

            addClass(commandClass);
        }

        doPostLoading(interceptor, postLoadCommands);

        FLog.info("Loaded " + commands.size() + " commands");
    }

    private void doPostLoading(Interceptor interceptor, Set<Class<? extends FreedomCommand>> postLoadCommands)
    {
        Bukkit.getScheduler().runTaskLater(plugin, () ->
                CompletableFuture.runAsync(interceptor::setOverrides)
                        .whenComplete((v, th) ->
                        {
                            if (th != null)
                            {
                                FLog.severe("Failed to set command overrides");
                                th.printStackTrace();
                                return;
                            }

                            for (Class<? extends FreedomCommand> commandClass : postLoadCommands)
                            {
                                if (checkDependencies(commandClass)) return;

                                addClass(commandClass);
                            }
                        }), 1);
    }

    private boolean checkDependencies(Class<? extends FreedomCommand> commandClass)
    {
        if (commandClass.isAnnotationPresent(CommandDependencies.class))
        {
            for (String dependency : commandClass.getAnnotation(CommandDependencies.class).value())
            {
                if (Bukkit.getPluginManager().getPlugin(dependency) == null)
                {
                    FLog.warning("Failed to register command: /" + commandClass.getSimpleName().replace("Command_", "") + " (Missing dependency: " + dependency + ")");
                    return true;
                }
            }
        }
        return false;
    }

    private void addClass(Class<? extends FreedomCommand> commandClass)
    {
        try
        {
            add(commandClass.newInstance());
        } catch (InstantiationException | IllegalAccessException ex)
        {
            FLog.warning("Failed to register command: /" + commandClass.getSimpleName().replace("Command_", ""));
        }
    }

    public List<FreedomCommand> getCommands()
    {
        return commands;
    }
}